[DEFAULT]

[processing]
{% if kolla_inspector_processing_hooks %}
# Comma-separated list of inspector processing plugins.
processing_hooks = {{ kolla_inspector_processing_hooks | join(',') }}
{% endif %}

{% if kolla_inspector_add_ports %}
# Which MAC addresses to add as ports during introspection. One of 'all',
# 'active' or 'pxe'.
add_ports = {{ kolla_inspector_add_ports }}
{% endif %}

{% if kolla_inspector_keep_ports %}
# Keep only those ports that were found during inspection.
keep_ports = present
{% endif %}

# Store logs returned by the inspection ramdisk.
always_store_ramdisk_logs = True

{% if kolla_inspector_enable_discovery %}
# Enable discovery when nodes do not exist in Ironic.
node_not_found_hook = enroll
{% endif %}

{% if kolla_inspector_enable_swift %}
store_data = swift
{% endif %}

{% if kolla_inspector_enable_swift %}
[swift]
{% for key, value in kolla_inspector_swift_auth.items() %}
{{ key }} = {{ value }}
{% endfor %}
{% endif %}

{% if kolla_inspector_enable_discovery %}
[discovery]
# The driver with which to enroll newly discovered nodes in Ironic.
enroll_node_driver = {{ kolla_inspector_discovery_enroll_node_driver }}
{% endif %}

{% if kolla_extra_inspector %}
#######################
# Extra configuration
#######################

{{ kolla_extra_inspector }}
{% endif %}
